#! /bin/bash

name=w32-cpython2.7-lib
badname=w32-cpython2.7-lib
ver=1.0
ver1=1
ver2=1.0
rev=5
subsys=mingw
srcname=${name}-${ver}
prefix=/mingw
CFLAGS="-g -O2"
docdir=share/doc/${badname}/${ver}
SOURCE_ARCHIVE_FORMAT=.zip
SOURCE_ARCHIVE=${name}-${ver}${SOURCE_ARCHIVE_FORMAT}
preconf_patches=
docfiles=""
licfiles=""

if test "x${install_dirs}" == "x"; then install_dirs="--datarootdir=\${prefix}/share --docdir=\${prefix}/\${docdir} --sbindir=\${prefix}/sbin --libexecdir=\${prefix}/libexec --sysconfdir=/etc --localstatedir=/var"; fi

url=http://you.cant.download.it.anywhere/${name}/${ver}/${SOURCE_ARCHIVE}

if test ! "$MSYSTEM" == "MSYS" -a "$subsys" == "msys"
then
  echo "You must be in an MSYS shell to build a msys package"
  exit 4
fi

if test ! "$MSYSTEM" == "MINGW32" -a "$subsys" == "mingw"
then
  echo "You must be in an MINGW shell to build a mingw package"
  exit 5
fi

pkgbuilddir=`pwd` || fail $LINENO
echo Acting from the directory ${pkgbuilddir}

patchfiles=`for i in ${pkgbuilddir}/patches/*.${subsys}.patch ${pkgbuilddir}/patches/*.all.patch; do if test $(expr index "$i" "*") -ge 1; then : ;else echo $i; fi; done | sort`
x=${pkgbuilddir}/${badname}
scriptfiles=`for i in $x-pre-install.sh $x-post-install.sh $x-pre-remove.sh $x-post-remove.sh; do if test ! -f $i; then : ;else echo $(basename $i); fi; done | sort`

instdir=${pkgbuilddir}/inst
blddir=${pkgbuilddir}/bld
logdir=${pkgbuilddir}/logs
#repodir=${pkgbuilddir}/repo
srcdir=${pkgbuilddir}/${name}-${ver}

echo The source directory is ${srcdir}

srcdirname=$(basename ${srcdir})

capname=Unknown
if test "x${subsys}" == "xmingw"
then
capname=MinGW
elif test "x${subsys}" == "xmsys"
then
capname=MSYS
fi

_sysver=$(uname -r)
export SYSVER=${_sysver%%\(*}


if test "x$arch" == "x" -a ! "x$buildalias" == "x"
then
  arch_index=$(expr index $buildalias -)
  arch_up_to=$((arch_index - 1))
  arch=${buildalias::${arch_up_to}}
fi
if test "x$arch" == "x"
then
  echo "Can't determine architecture."
  echo "Set 'arch' or 'buildalias' environment variable."
  exit 1
fi

p1=${prefix:1}

do_clone=0
do_download=0
do_unpack=0
do_copy=0
do_patch=0
do_preconfigure=0
do_reconfigure=0
do_configure=0
do_make=0
do_check=0
do_install=1
do_fixinstall=1
do_pack=1
do_clean=1

if test ! "x$repodir" = "x"
then
  do_clone=1
  do_download=0
  do_unpack=0
  do_copy=1
  do_reconfigure=1
fi

zeroall() {
      do_clone=0
      do_download=0
      do_unpack=0
      do_copy=0
      do_patch=0
      do_preconfigure=0
      do_reconfigure=0
      do_configure=0
      do_make=0
      do_check=0
      do_install=0
      do_fixinstall=0
      do_pack=0
      do_clean=0
}

while [ $# -gt 0 ]
do
  case $1 in
    --clone) do_clone=1 ; shift 1 ;;
    --copy) do_copy=1 ; shift 1 ;;
    --download) do_download=1 ; shift 1 ;;
    --unpack) do_unpack=1 ; shift 1 ;;
    --patch) do_patch=1 ; shift 1 ;;
    --preconfigure) do_preconfigure=1 ; shift 1 ;;
    --reconfigure) do_reconfigure=1 ; shift 1 ;;
    --configure) do_configure=1 ; shift 1 ;;
    --make) do_make=1 ; shift 1 ;;
    --check) do_check=1 ; shift 1 ;;
    --install) do_install=1 ; shift 1 ;;
    --fixinstall) do_fixinstall=1 ; shift 1 ;;
    --pack) do_pack=1 ; shift 1 ;;
    --clone-only) zeroall; do_clone=1 ; shift 1 ;;
    --copy-only) zeroall; do_copy=1 ; shift 1 ;;
    --download-only) zeroall; do_download=1 ; shift 1 ;;
    --unpack-only) zeroall; do_unpack=1 ; shift 1 ;;
    --patch-only) zeroall; do_patch=1 ; shift 1 ;;
    --preconfigure-only) zeroall; do_preconfigure=1 ; shift 1 ;;
    --reconfigure-only) zeroall; do_reconfigure=1 ; shift 1 ;;
    --configure-only) zeroall; do_configure=1 ; shift 1 ;;
    --make-only) zeroall; do_make=1 ; shift 1 ;;
    --check-only) zeroall; do_check=1 ; shift 1 ;;
    --install-only) zeroall; do_install=1 ; shift 1 ;;
    --fixinstall-only) zeroall; do_fixinstall=1 ; shift 1 ;;
    --pack-only) zeroall; do_pack=1 ; shift 1 ;;
    --dont-clean) do_clean=0 ; shift 1 ;;
    *) shift 1 ;;
  esac
done


fail() {
  echo "failure at line $1"
  exit 1
}

if test ! -d ${logdir}
then
  mkdir -p ${logdir} || fail $LINENO
fi

install=/bin/install
if test "x$do_install" == "x1"
then
  rm -f ${logdir}/install.log

  echo Cleaning up inst directory | tee -a ${logdir}/install.log
  rm -rf ${instdir} 2>&1 | tee -a ${logdir}/install.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  echo Installing into ${instdir} | tee ${logdir}/install.log
  ${install} -d ${instdir}
  cp -R ${pkgbuilddir}/mingw ${instdir}/mingw
  ln -s python2.7-config ${instdir}/mingw/bin/python-config
fi

if test "x$do_pack" == "x1"
then
  rm -f ${logdir}/pack.log
  echo Packing | tee -a ${logdir}/pack.log
  packed=${pkgbuilddir}/packed
  installed=${pkgbuilddir}/installed
  unused=${pkgbuilddir}/unused
  rm -f ${packed}
  cd ${instdir}  && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  pkgtmp=${ver}-${rev}-${subsys}_${arch}

  tar cv --xz -f ${pkgbuilddir}/${name}-${pkgtmp}-bin.tar.xz \
$p1 \
  2>&1 | tee -a ${logdir}/pack.log | tee -a ${packed} && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  uniq <${packed} | sort  | while read p; do if test -f "${instdir}/$p" -o -L "${instdir}/$p"; then echo "$p"; fi; done >${packed}.files
  find ${instdir} ! -type d -printf '%P\n' | uniq | sort >${installed}
  diff -u0 ${installed} ${packed}.files | tail -n+3 | sed -e 's/^-\(.\+\)$/\1/' -e '/^@@ .\+ @@$/d' >${unused}
  if test x0 != x$(wc -l < ${unused}); then
    echo "Unused files:"
    cat ${unused}
    echo ""
  fi
  rm -f ${installed} ${packed} ${packed}.files ${unused}

  cd ${pkgbuilddir}  && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  BLD="\
pkgbuild.sh \
mingw \
"
  tar cv --xz -f ${pkgbuilddir}/${badname}-${pkgtmp}-bld.tar.xz $BLD 2>&1 | tee -a ${logdir}/pack.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
fi

echo "Done"
exit 0
