#! /bin/bash

name=python
badname=python2.7
ver=2.7.9
ver1=2
ver2=2.7
rev=1
subsys=msys2
srcname=${name}-${ver}
prefix=/usr
CFLAGS="-g -O2"
docdir=share/doc/${badname}/${ver}
SOURCE_ARCHIVE_FORMAT=.tar.xz
SOURCE_ARCHIVE=Python-${ver}${SOURCE_ARCHIVE_FORMAT}
preconf_patches=
docfiles="README"
licfiles="LICENSE"

if test "x${install_dirs}" == "x"; then install_dirs="--datarootdir=\${prefix}/share --docdir=\${prefix}/\${docdir} --sbindir=\${prefix}/sbin --libexecdir=\${prefix}/libexec --sysconfdir=/etc --localstatedir=/var"; fi
#if test "x${install_dirs}" == "x"; then install_dirs="--datadir=\${prefix}/share --infodir=\${prefix}/share/info --mandir=\${prefix}/share/man --sbindir=\${prefix}/sbin --libexecdir=\${prefix}/libexec --sysconfdir=\${prefix}/etc --localstatedir=/var"; fi


reposrc=""

url=http://www.python.org/ftp/python/${ver}/${SOURCE_ARCHIVE}

if test ! "$MSYSTEM" == "MSYS" -a "$subsys" == "msys2"
then
  echo "You must be in an MSYS shell to build a msys package"
  exit 4
fi

if test ! "$MSYSTEM" == "MINGW32" -a "$subsys" == "mingw32"
then
  echo "You must be in an MINGW shell to build a mingw32 package"
  exit 5
fi

pkgbuilddir=`pwd` || fail $LINENO
echo Acting from the directory ${pkgbuilddir}

patchfiles=`for i in ${pkgbuilddir}/patches/*.${subsys}.patch ${pkgbuilddir}/patches/*.all.patch; do if test $(expr index "$i" "*") -ge 1; then : ;else echo $i; fi; done | sort`
x=${pkgbuilddir}/${badname}
scriptfiles=`for i in $x-pre-install.sh $x-post-install.sh $x-pre-remove.sh $x-post-remove.sh; do if test ! -f $i; then : ;else echo $(basename $i); fi; done | sort`

instdir=${pkgbuilddir}/inst
blddir=${pkgbuilddir}/bld
logdir=${pkgbuilddir}/logs
srcdir=${pkgbuilddir}/${name}-${ver}

echo The source directory is ${srcdir}

srcdirname=$(basename ${srcdir})

capname=Unknown
if test "x${subsys}" == "xmingw32"
then
capname=MinGW
elif test "x${subsys}" == "xmsys2"
then
capname=MSYS
fi

_sysver=$(uname -r)
export SYSVER=${_sysver%%\(*}

if test "x$arch" == "x" -a ! "x$buildalias" == "x"
then
  arch_index=$(expr index $buildalias -)
  arch_up_to=$((arch_index - 1))
  arch=${buildalias::${arch_up_to}}
fi
if test "x$arch" == "x"
then
  echo "Can't determine architecture."
  echo "Set 'arch' or 'buildalias' environment variable."
  exit 1
fi

p1=${prefix:1}

do_download=1
do_unpack=1
do_patch=1
do_preconfigure=0
do_reconfigure=1
do_configure=1
do_make=1
do_check=0
do_install=1
do_fixinstall=1
do_pack=1
do_clean=1

if test ! "x$reposrc" = "x"
then
  srcdir="$reposrc"
  do_download=0
  do_unpack=0
  do_reconfigure=1
fi

zeroall() {
      do_download=0
      do_unpack=0
      do_patch=0
      do_preconfigure=0
      do_reconfigure=0
      do_configure=0
      do_make=0
      do_check=0
      do_install=0
      do_fixinstall=0
      do_pack=0
      do_clean=0
}

while [ $# -gt 0 ]
do
  case $1 in
    --download) do_download=1 ; shift 1 ;;
    --unpack) do_unpack=1 ; shift 1 ;;
    --patch) do_patch=1 ; shift 1 ;;
    --preconfigure) do_preconfigure=1 ; shift 1 ;;
    --reconfigure) do_reconfigure=1 ; shift 1 ;;
    --configure) do_configure=1 ; shift 1 ;;
    --make) do_make=1 ; shift 1 ;;
    --check) do_check=1 ; shift 1 ;;
    --install) do_install=1 ; shift 1 ;;
    --fixinstall) do_fixinstall=1 ; shift 1 ;;
    --pack) do_pack=1 ; shift 1 ;;
    --download-only) zeroall; do_download=1 ; shift 1 ;;
    --unpack-only) zeroall; do_unpack=1 ; shift 1 ;;
    --patch-only) zeroall; do_patch=1 ; shift 1 ;;
    --preconfigure-only) zeroall; do_preconfigure=1 ; shift 1 ;;
    --reconfigure-only) zeroall; do_reconfigure=1 ; shift 1 ;;
    --configure-only) zeroall; do_configure=1 ; shift 1 ;;
    --make-only) zeroall; do_make=1 ; shift 1 ;;
    --check-only) zeroall; do_check=1 ; shift 1 ;;
    --install-only) zeroall; do_install=1 ; shift 1 ;;
    --fixinstall-only) zeroall; do_fixinstall=1 ; shift 1 ;;
    --pack-only) zeroall; do_pack=1 ; shift 1 ;;
    --dont-clean) do_clean=0 ; shift 1 ;;
    *) shift 1 ;;
  esac
done


fail() {
  echo "failure at line $1"
  exit 1
}

if test ! -d ${logdir}
then
  mkdir -p ${logdir} || fail $LINENO
fi

if test "x$do_download" == "x1"
then
  rm -f ${logdir}/download.log
  test -f ${pkgbuilddir}/${SOURCE_ARCHIVE} || {
    echo "Downloading ${SOURCE_ARCHIVE} ..." 2>&1 | tee ${logdir}/download.log
    wget -P ${pkgbuilddir} $url 2>&1 | tee -a ${logdir}/download.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
    b=$(basename $url)
    if test ! "x$b" = "x${SOURCE_ARCHIVE}"
    then
      mv -f $b ${SOURCE_ARCHIVE}
    fi
  }

  if test -f ${SOURCE_ARCHIVE}.sha512
  then
    echo "Verifying sha512 sum ..." | tee -a ${logdir}/download.log
    sha512sum -c ${SOURCE_ARCHIVE}.sha512 2>&1 | tee -a ${logdir}/download.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
  else
    echo "Calculating sha512 sum ..." | tee -a ${logdir}/download.log
    sha512sum ${SOURCE_ARCHIVE} >${SOURCE_ARCHIVE}.sha512 2>>${logdir}/download.log || fail $LINENO
  fi
  echo "Done downloading"
fi

if test "x$do_unpack" == "x1"
then
  rm -f ${logdir}/unpack.log
  if test ! "x${srcdir}" == "x/" -a ! "x${srcdir}" == "x" -a ! "x${srcdir}" == "x${pkgbuilddir}"
  then
    echo "Deleting ${srcdir} contents" | tee ${logdir}/unpack.log
    rm -rf ${srcdir}/* 2>&1 | tee -a ${logdir}/unpack.log
  else
    echo "I think it is unsafe to delete ${srcdir}" | tee ${logdir}/unpack.log
    exit 2
  fi

  echo Cleaning up inst and build directories | tee -a ${logdir}/unpack.log
  rm -rf ${blddir} ${instdir} 2>&1 | tee -a ${logdir}/unpack.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  cd ${pkgbuilddir} && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  echo "Unpacking $SOURCE_ARCHIVE in `pwd`" | tee -a ${logdir}/unpack.log
  case "$SOURCE_ARCHIVE" in
  *.tar.bz2 ) tar xjf $SOURCE_ARCHIVE 2>&1 | tee -a ${logdir}/unpack.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi;;
  *.tar.gz  ) tar xzf $SOURCE_ARCHIVE 2>&1 | tee -a ${logdir}/unpack.log  && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi;;
  *.zip     ) unzip -q $SOURCE_ARCHIVE  2>&1 | tee -a ${logdir}/unpack.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi;;
  *.tar.lzma) tar --lzma -xf $SOURCE_ARCHIVE 2>&1 | tee -a ${logdir}/unpack.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi;;
  *.tar.xz) tar xJf $SOURCE_ARCHIVE 2>&1 | tee -a ${logdir}/unpack.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi;;
  esac
  if test $(ls ${srcdir} -l | wc -l) -ge 2
  then
    echo "Unpacked into ${srcdir} successfully!" | tee -a ${logdir}/unpack.log
  else
    echo "${srcdir} is empty - did not unpack into it?" | tee -a ${logdir}/unpack.log
    exit 3
  fi
  echo "Done unpacking"
fi

patch_list=
if test "x$do_patch" == "x1"
then
  rm -f ${logdir}/patch.log
  echo cd ${srcdir}
  cd ${srcdir} && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  if test ! "x${patchfiles}" == "x"
  then
    echo "Patching in `pwd` from patchfiles ${patchfiles}" | tee -a ${logdir}/patch.log
  fi
  for patchfile in ${patchfiles}
  do
    if test ! "x${patchfiles}" == "x"
    then
      echo "Applying ${patchfile}" | tee -a ${logdir}/patch.log
      patch -p1 -i ${patchfile} 2>&1 | tee -a ${logdir}/patch.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
    fi
    patch_list="$patch_list $patchfile"
  done
  echo "Done patching"
fi

if test "x$do_reconfigure" == "x1"
then
  cd ${srcdir}  && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  echo Reconfiguring in `pwd` | tee -a ${logdir}/reconfigure.log
  libtoolize -fi 2>&1 | tee -a ${logdir}/reconfigure.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
  autoreconf -fi 2>&1 | tee -a ${logdir}/reconfigure.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
elif test "x$do_preconfigure" == "x1"
then
  cd ${srcdir}  && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  echo Patching in `pwd` from ${preconf_patches} | tee -a ${logdir}/reconfigure.log
  for patchfile in ${pkgbuilddir}/preconf-patches/*${subsys}.patch
  do
    patch -p0 -i ${patchfile} 2>&1 | tee -a ${logdir}/reconfigure.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
  done
fi

mkdir -p ${blddir} || fail $LINENO

if test "x$do_configure" == "x1"
then
  rm -f ${logdir}/configure.log
  cd ${blddir} && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  eval "install_dirs=\"$install_dirs\""

  build_host_target=
  if test ! "x${buildalias}" == "x"
  then
    build_host_target+=" --build=${buildalias}"
  fi
  if test ! "x${hostalias}" == "x"
  then
    build_host_target+=" --host=${hostalias}"
  fi
  if test ! "x${targetalias}" == "x"
  then
    build_host_target+=" --target=${targetalias}"
  fi

  echo Configuring in `pwd` | tee -a ${logdir}/configure.log
  ../$(basename ${srcdir})/configure \
    --prefix=${prefix} \
    --enable-shared \
    --enable-ipv6 \
    --with-dbmliborder=gdbm \
    --with-libc= \
    --with-libm= \
    --with-system-ffi \
    --with-system-expat \
    ac_cv_func_bind_textdomain_codeset=yes \
    ${build_host_target} \
    ${install_dirs} \
    CFLAGS="$CFLAGS" STRIP=true \
    CPPFLAGS="$CPPFLAGS -I${prefix}/include/ncursesw" \
    2>&1 | tee -a ${logdir}/configure.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
  
fi

if test "x$do_make" == "x1"
then
  rm -f ${logdir}/make.log
  cd ${blddir}  && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  echo Making
  make $MAKEFLAGS -j1 2>&1 | tee -a ${logdir}/make.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
fi

if test "x$do_install" == "x1"
then
  rm -f ${logdir}/install.log
  cd ${blddir}  && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  echo Installing into ${instdir} from `pwd` | tee ${logdir}/install.log
  make altinstall maninstall DESTDIR=${instdir} $MAKEFLAGS -j1 2>&1 | tee -a ${logdir}/install.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi 

  cd ${pkgbuilddir}
  if test ! "x${scriptfiles}" == "x"
  then
    mkdir -p ${instdir}/var/lib/mingw-get/scripts
  fi
  for scriptfile in ${scriptfiles}
  do
    if test ! "x${scriptfile}" == "x"
    then
      cp ${scriptfile} ${instdir}/var/lib/mingw-get/scripts/$(echo ${scriptfile} | sed -e 's/'${badname}'-/'${badname}'-'${ver}'-'${rev}'-/')
    fi
  done
fi

if test "x$do_check" == "x1"
then
  rm -f ${logdir}/test.log
  cd ${blddir}  && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  echo "Testing" | tee -a ${logdir}/test.log
  make -k check 2>&1 | tee -a ${logdir}/test.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
fi


if test "x$do_fixinstall" == "x1"
then
  rm -f ${logdir}/fixinstall.log
  echo Fixing the installation | tee ${logdir}/fixinstall.log
  mkdir -p ${instdir}${prefix}/share/doc/${badname}/${ver} 2>&1 | tee -a ${logdir}/fixinstall.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
  cd ${srcdir}  && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
  for f in ${docfiles} ${licfiles}
  do
    if test -f ${instdir}${prefix}/${docdir}/$(basename ${f})
    then
      cp -r ${f} ${instdir}${prefix}/${docdir}/${f}
    else
      cp -r ${f} ${instdir}${prefix}/${docdir}/$(basename ${f})
    fi
  done
  mkdir -p ${instdir}${prefix}/share/doc/${capname} 2>&1 | tee -a ${logdir}/fixinstall.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
  cp ${pkgbuilddir}/${subsys}-${badname}.RELEASE_NOTES \
    ${instdir}${prefix}/share/doc/${capname}/${badname}-${ver}-${rev}-${subsys}.RELEASE_NOTES.txt 2>&1 | tee -a ${logdir}/fixinstall.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  find ${instdir}${prefix}/bin/ -type f | while read p; do if test "x$(head -c 2 $p)" = "x#!"; then echo $p; fi; done | xargs sed -i -e "0,/^#!/{s|^#!.*|#! ${prefix}/bin/python${ver2}|}"

  ln -s python${ver2}.exe ${instdir}${prefix}/bin/python
  ln -s python${ver2}.1 ${instdir}${prefix}/share/man/man1/python.1
  ln -s python${ver2}-config ${instdir}${prefix}/bin/python-config

  ln -s python${ver2}/config/libpython${ver2}.dll.a ${instdir}${prefix}/lib/libpython${ver2}.dll.a
  chmod u+w ${instdir}${prefix}/bin/libpython${ver2}.dll
  /usr/bin/install -d ${instdir}/etc/profile.d
  /usr/bin/install ${pkgbuilddir}/python2-utf8.sh ${instdir}/etc/profile.d/python2-utf8.sh

  /mingw/bin/laremove.py $(cd ${instdir}; pwd -W) && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
  /mingw/bin/split-debug.py $(cd ${instdir}; pwd -W) --target=$buildalias && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
fi

if test "x$do_pack" == "x1"
then
  rm -f ${logdir}/pack.log
  packed=${pkgbuilddir}/packed
  installed=${pkgbuilddir}/installed
  unused=${pkgbuilddir}/unused
  rm -f ${packed}
  cd ${instdir}  && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
  echo Packing | tee -a ${logdir}/pack.log

  pkgtmp=${ver}-${rev}-${subsys}_${arch}

  tar cv --xz -f ../python2.7-${pkgtmp}-bin.tar.xz \
--exclude=*.dbg \
$p1/bin/* \
$p1/lib/* \
$p1/include/* \
etc \
  2>&1 | tee -a ${logdir}/pack.log | tee -a ${packed} && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  tar cv --xz -f ../python2.7-${pkgtmp}-dbg.tar.xz \
$p1/bin/python${ver2}.exe.dbg \
$p1/bin/libpython${ver2}.dll.dbg \
$p1/lib/python${ver2}/distutils/command/wininst-6.0.exe.dbg \
$p1/lib/python${ver2}/distutils/command/wininst-7.1.exe.dbg \
$p1/lib/python${ver2}/distutils/command/wininst-8.0.exe.dbg \
$p1/lib/python${ver2}/distutils/command/wininst-9.0.exe.dbg \
$p1/lib/python${ver2}/lib-dynload/array.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/audioop.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/binascii.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/bz2.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/cmath.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/cPickle.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/crypt.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/cStringIO.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/datetime.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/dl.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/fcntl.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/future_builtins.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/gdbm.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/grp.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/imageop.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/itertools.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/math.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/mmap.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/operator.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/parser.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/pyexpat.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/readline.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/resource.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/select.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/strop.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/syslog.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/termios.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/time.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/unicodedata.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/zlib.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_bisect.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_bsddb.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_codecs_cn.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_codecs_hk.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_codecs_iso2022.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_codecs_jp.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_codecs_kr.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_codecs_tw.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_collections.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_csv.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_ctypes.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_ctypes_test.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_curses.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_curses_panel.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_elementtree.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_functools.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_hashlib.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_heapq.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_hotshot.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_io.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_json.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_locale.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_lsprof.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_multibytecodec.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_multiprocessing.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_random.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_socket.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_sqlite3.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_ssl.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_struct.dll.dbg \
$p1/lib/python${ver2}/lib-dynload/_testcapi.dll.dbg \
  2>&1 | tee -a ${logdir}/pack.log | tee -a ${packed} && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  tar cv --xz -f ${pkgbuilddir}/${badname}-${pkgtmp}-doc.tar.xz \
$p1/share/man \
$p1/${docdir} \
$p1/share/doc/${capname} \
 2>&1 | tee -a ${logdir}/pack.log | tee -a ${packed} && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  uniq <${packed} | sort  | while read p; do if test -f ${instdir}/$p -o -L ${instdir}/$p; then echo $p; fi; done >${packed}.files
  find ${instdir} ! -type d -printf '%P\n' | uniq | sort >${installed}
  diff -u0 ${installed} ${packed}.files | tail -n+3 | sed -e 's/^-\(.\+\)$/\1/' -e '/^@@ .\+ @@$/d' >${unused}
  if test x0 != x$(wc -l < ${unused}); then
    echo "Unused files:"
    cat ${unused}
    echo ""
  fi
  rm -f ${installed} ${packed} ${packed}.files ${unused}

  cd ${pkgbuilddir}  && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  BLD="\
pkgbuild.sh \
python2-utf8.sh \
${subsys}-${badname}.RELEASE_NOTES \
$(if test -d patches; then echo patches; fi) \
${scriptfiles} \
${SOURCE_ARCHIVE}.sha512 \
"

  tar cv --xz -f ${pkgbuilddir}/${badname}-${pkgtmp}-bld.tar.xz $BLD 2>&1 | tee -a ${logdir}/pack.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi

  tar cv --xz -f ${pkgbuilddir}/${badname}-${pkgtmp}-src.tar.xz ${SOURCE_ARCHIVE} $BLD 2>&1 | tee -a ${logdir}/pack.log && if ! test "x${PIPESTATUS[0]}" == "x0"; then fail $LINENO; fi
fi

if test "x$do_clean" == "x1"
then

cd ${pkgbuilddir}
  rm -rf ${blddir} ${instdir}
  if test -d "${srcdir}" -a ! "x${srcdir}" == "x/" -a ! "x${srcdir}" == "x" -a ! "x${srcdir}" == "x${pkgbuilddir}"
  then
    rm -rf ${srcdir}
  fi
fi

echo "Done"
exit 0
